/* eslint-env node */
'use strict';

/**
 * Please *do not* import this file in your project. We put this here for
 * backwards compatibility reasons with the ember-decorators main package.
 *
 * This is explicitly private API and not intended for use inside third-party
 * addons.
 *
 * If you find yourself in need of importing this file, because babel-transforms
 * does not serve your special use case, please open an issue instead and let us
 * find a solution. :)
 */

const VersionChecker = require('ember-cli-version-checker');

function requireTransform(transformName) {
  return require.resolve(transformName);
}

function hasPlugin(plugins, name) {
  for (const maybePlugin of plugins) {
    const plugin = Array.isArray(maybePlugin) ? maybePlugin[0] : maybePlugin;
    const pluginName = typeof plugin === 'string' ? plugin : plugin.name;

    if (pluginName === name) {
      return true;
    }
  }

  return false;
}

module.exports = function setupBabel(parent) {
  // Create parent options, if they do not exist
  const parentOptions = (parent.options = parent.options || {});

  const checker = new VersionChecker(parent).for('ember-cli-babel', 'npm');

  // find out why this doesn't work
  if (true || checker.satisfies('^7.0.0-beta.3')) {
    // Create babel options if they do not exist
    parentOptions.babel = parentOptions.babel || {};

    // Create and pull off babel plugins
    const plugins = (parentOptions.babel.plugins =
      parentOptions.babel.plugins || []);

    if (!hasPlugin(plugins, '@babel/plugin-proposal-decorators')) {
      // unshift the transform because it always must come before class properties
      plugins.unshift([requireTransform('@babel/plugin-proposal-decorators'), { legacy: true }]);
    }

    if (!hasPlugin(plugins, '@babel/transform-typescript')) {
      // unshift the transform because it always must come before decorators
      plugins.unshift(requireTransform('@babel/plugin-transform-typescript'));
    }

    if (!hasPlugin('@babel/plugin-proposal-class-properties')) {
      plugins.push([requireTransform('@babel/plugin-proposal-class-properties'), { loose: true }]);
    }

    if (!hasPlugin('@babel/proposal-object-rest-spread')) {
      plugins.push(requireTransform('@babel/plugin-proposal-object-rest-spread'));
    }

    // Create babel options if they do not exist
    const emberCliBabel = (parentOptions['ember-cli-babel'] =
      parentOptions['ember-cli-babel'] || {});

    if (emberCliBabel.extensions) {
      emberCliBabel.extensions = ['ts'].concat(emberCliBabel.extensions);
    } else {
      emberCliBabel.extensions = ['ts', 'js'];
    }
  } else {
    parent.project.ui.writeWarnLine(
      'ember-cli-babel-typescript: You are using an unsupported ember-cli-babel version, TypeScript will not be enabled.'
    );
  }
};
